// 2003 Theo Lustenberger
unit SelectionRectangle;

interface

uses QGraphics, Types, classes, qextctrls, qcontrols;

type TActionType = (atMove, atSelect, atResize, atLine);
type TResizeStyle = (rssLeft, rssTop, rssRight, rssBottom,
    rssBottomRight, rssBottomLeft, rssTopRight, rssTopLeft,
    rssMove, rssNone);

type
  TSelectionRectangle = class(TObject)
  private
    fCanvas: TCanvas;
    fTim: TTimer;
    fTimerToggle: boolean;
    fRct: TRect;
    fmovePoint: TPoint;
    fStartmovePoint: TPoint;
    fSavedMovePoint: TPoint;
    fSavedRct: TRect;
    fSelecting: Boolean;
    fActionType: TActionType;
    fLastActionType: TActionType;
    fResizeStyle: TResizeStyle;
    procedure TimerEvent(Sender: TObject);
    procedure EnableTimer;
    procedure DisableTimer;
    procedure TimerProc;
    procedure SetRct(Rectangle: TRect);
    function GetRct: TRect;
    procedure InitSelect;
    procedure InitLine;
    procedure Selecting;
    procedure StopSelect;
    procedure StopMove;
    procedure MoveSelect;
  protected

  public
    constructor Create(Canvas: TCanvas);
    destructor Destroy; override;
    procedure StartLine(StartPoint: TPoint);
    procedure StartSelect(StartPoint: TPoint);
    procedure StartMove(StartPoint: TPoint);
    procedure StartResize(StartPoint: TPoint; ResizeStyle: TResizeStyle);
    procedure MouseMove(CurrentPoint: TPoint);
    procedure MouseUp;
    procedure Reset;
    procedure ResetRotate;
    procedure Hide;
    procedure Show;
    property Rct: TRect read GetRct write SetRct;
    procedure Draw(NewImage: TBitmap; w, h: integer; DoReset: Boolean);
    function GetResizeStyle(X, Y, Tolerance: integer): TResizeStyle;
    function GetResizeCursor(X, Y, Tolerance: integer): TCursor;
    function AdjustRect(theRect: TRect): TRect;
    function InsideRect(X, Y: integer; theRect: TRect; Tolerance: integer): boolean;
    

  published

  end;

  var SelectedRect: TRect;


implementation

uses Sysutils;

constructor TSelectionRectangle.Create(Canvas: TCanvas);
begin
  fCanvas :=Canvas;
  Canvas.Pen.Width:=1;
  fTim := TTimer.create(nil);
  fTim.Enabled := false;
  fTim.Interval := 600;
  ftimerToggle := true;
  fTim.OnTimer := TimerEvent;
  fRct := Rect(0, 0, 0, 0);
  fSavedRct := fRct;
end;

destructor TSelectionRectangle.Destroy;
begin
  fTim.free;
end;

{***********************************************************}

procedure TSelectionRectangle.SetRct(Rectangle: TRect);
begin
  if fSelecting then fRct := Rectangle;
end;

function TSelectionRectangle.GetRct: TRect;
begin
  Result := AdjustRect(fRct);
end;

procedure TSelectionRectangle.MouseMove(CurrentPoint: TPoint);
begin

  if fSelecting then
  begin
    if fActionType = atSelect then
    begin
      fRct.Right := CurrentPoint.x;
      fRct.Bottom := CurrentPoint.y;
      Selecting;
    end;

    if fActionType = atLine then
    begin
      fRct.Right := CurrentPoint.x;
      fRct.Bottom := CurrentPoint.y;
      fcanvas.MoveTo(fSavedRct.Left, fSavedRct.Top);
      fcanvas.Lineto(fSavedRct.Right, fSavedRct.Bottom);
      fcanvas.MoveTo(fRct.Left, fRct.Top);
      fcanvas.Lineto(fRct.Right, fRct.Bottom);
      fSavedRct := fRct;
    end;

    if fActionType = atMove then
    begin
      fMovePoint := CurrentPoint;
      MoveSelect;

    end;

    if fActionType = atResize then
    begin
      case fResizeStyle of
        rssRight: fRct.Right := CurrentPoint.x;
        rssLeft: fRct.Left := CurrentPoint.x;
        rssTop: fRct.Top := CurrentPoint.y;
        rssBottom: fRct.Bottom := CurrentPoint.y;

        rssBottomRight:
          begin
            fRct.Bottom := CurrentPoint.y;
            fRct.Right := CurrentPoint.x;
          end;

        rssBottomLeft:
          begin
            fRct.Bottom := CurrentPoint.y;
            fRct.Left := CurrentPoint.x;
          end;

        rssTopRight:
          begin
            fRct.Top := CurrentPoint.y;
            fRct.Right := CurrentPoint.x;
          end;

        rssTopLeft:
          begin
            fRct.Top := CurrentPoint.y;
            fRct.Left := CurrentPoint.x;
          end;

      end;
      Selecting;
    end;

  end;

end;

procedure TSelectionRectangle.MouseUp;
begin
  fLastActionType := fActionType;
  if FSelecting then
  begin


    if fActionType = atLine then
    begin
      fSelecting := false;
    end;

    if fActionType = atSelect then
    begin
      StopSelect;
    end;

    if fActionType = atMove then
    begin
      StopMove;

    end;

    if fActionType = atResize then
    begin
      StopSelect;
    end;



  end;

end;



procedure TSelectionRectangle.InitSelect;
begin
  fCanvas.pen.style := psDot;
  fCanvas.pen.mode := pmNotXor;
  fCanvas.pen.width := 1;
  //fCanvas.pen.Color:=$007FAA00;
  fCanvas.pen.Color:=clBlack;

end;

procedure TSelectionRectangle.InitLine;
begin
  fCanvas.pen.style := psSolid;
  fCanvas.pen.mode := pmNotXor;
  fCanvas.pen.width := 1;
end;

{***********************************************************}

procedure TSelectionRectangle.StartLine(StartPoint: TPoint);
begin

  Reset;
  if not fSelecting then
  begin
    fActionType := atLine;
    InitLine;
    fSelecting := true;
    fRct := Rect(StartPoint.x, StartPoint.y, StartPoint.x, StartPoint.y);
  end;

end;


procedure TSelectionRectangle.StartSelect(StartPoint: TPoint);
begin

  Reset;
  if not fSelecting then
  begin
    fActionType := atSelect;
    InitSelect;
    fSelecting := true;
    fRct := Rect(StartPoint.x, StartPoint.y, StartPoint.x, StartPoint.y);
  end;

end;

procedure TSelectionRectangle.Selecting;
var SelWidth: integer;
    SelHeight: integer;
begin

  if fSelecting then
  begin
    fcanvas.rectangle(fSavedRct.Left, fSavedRct.Top, fSavedRct.Right, fSavedRct.Bottom);
    fcanvas.rectangle(fRct.Left, fRct.Top, fRct.Right, fRct.Bottom);
    SelWidth:=Abs(fRct.Right-fRct.Left);
    SelHeight:=Abs(fRct.Bottom-fRct.Top);

    //PaintForm.StatusBar.Panels[1].Text := Format('%d,%d', [SelWidth,SelHeight]);

    fSavedRct := fRct;
  end;

end;

procedure TSelectionRectangle.StopSelect;
begin

  if fSelecting then
  begin
    //EnableTimer;
    fSelecting := false;
    //SelectedRect:=rect(Sel.Rct.Left-1,Sel.Rct.Top-1,Sel.Rct.Right-2,Sel.Rct.Bottom-2);
  end;

end;

{***********************************************************}

procedure TSelectionRectangle.StartMove(StartPoint: TPoint);
begin

  if not fSelecting then
  begin
    fActionType := atMove;
    fMovePoint := StartPoint;
    fStartMovePoint := StartPoint;
    fSavedMovePoint := StartPoint;
    //DisableTimer;
    fSelecting := true;
  end;

end;

procedure TSelectionRectangle.MoveSelect;
begin

  if fSelecting then
  begin
    fcanvas.rectangle(fRct.Left-1 - fStartMovePoint.x + fSavedMovePoint.x,
                      fRct.Top-1 - fStartMovePoint.y + fSavedMovePoint.y,
                      fRct.Right - fStartMovePoint.x + fSavedMovePoint.x,
                      fRct.Bottom - fStartMovePoint.y + fSavedMovePoint.y);


     { if MoveMode=mmCut then begin
         PaintForm.ImageWork.Canvas.Brush.Color:=PaintForm.pnlColorBG.Color;
         PaintForm.ImageWork.Canvas.Pen.Color:=PaintForm.pnlColorBG.Color;
         PaintForm.ImageWork.Canvas.Brush.Style:=bsSolid;
         PaintForm.ImageWork.Canvas.rectangle(SelectedRect);
         PaintForm.PaintBox.Canvas.Draw(0,0,PaintForm.ImageWork.Picture.Graphic);
      end;
       PaintForm.PaintBox.Canvas.Draw(0,0,PaintForm.ImageWork.Picture.Graphic);

      if TransparenceMode= tmSolid then PaintForm.PaintBox.Canvas.CopyMode:=cmSrcCopy;
      if TransparenceMode = tmTransparent then PaintForm.PaintBox.Canvas.CopyMode:=cmMergeCopy;

      

      PaintForm.PaintBox.Canvas.CopyRect(rect(fRct.Left+2 - fStartMovePoint.x + fMovePoint.x,
                                              fRct.Top+2 - fStartMovePoint.y + fMovePoint.y,
                                              fRct.Right - fStartMovePoint.x + fMovePoint.x,
                                              fRct.Bottom - fStartMovePoint.y + fMovePoint.y),
                                              PaintForm.ImageBack.Canvas,
                                              SelectedRect);

     PaintForm.PaintBox.Canvas.CopyMode:=cmSrcCopy;}

    fcanvas.rectangle(fRct.Left-1 - fStartMovePoint.x + fMovePoint.x,
                      fRct.Top-1 - fStartMovePoint.y + fMovePoint.y,
                      fRct.Right - fStartMovePoint.x + fMovePoint.x,
                      fRct.Bottom - fStartMovePoint.y + fMovePoint.y);

    fSavedMovePoint := fMovePoint;


  end;

end;


procedure TSelectionRectangle.StopMove;
begin

  if fSelecting then
  begin
    fRct := Rect(fRct.Left - fStartMovePoint.x + fMovePoint.x,
      fRct.Top - fStartMovePoint.y + fMovePoint.y,
      fRct.Right - fStartMovePoint.x + fMovePoint.x,
      fRct.Bottom - fStartMovePoint.y + fMovePoint.y);
    fSavedRct := fRct;
    //EnableTimer;

      {if TransparenceMode= tmSolid then PaintForm.ImageBack.Canvas.CopyMode:=cmSrcCopy;
      if TransparenceMode = tmTransparent then PaintForm.ImageBack.Canvas.CopyMode:=cmMergeCopy;

      PaintForm.ImageWork.Canvas.CopyMode:=PaintForm.ImageBack.Canvas.CopyMode;
      PaintForm.ImageWork.Canvas.CopyRect(fRct,PaintForm.ImageBack.Canvas,SelectedRect);
      PaintForm.ImageBack.Picture:=PaintForm.ImageWork.Picture; }

    fSelecting := false;
  end;


end;

{***********************************************************}

procedure TSelectionRectangle.StartResize(StartPoint: TPoint; ResizeStyle: TResizeStyle);
begin

  if not fSelecting then
  begin
    fActionType := atResize;
    fResizeStyle := ResizeStyle;
    fSelecting := true;
    //DisableTimer;

    //PaintForm.PaintBox.Canvas.Draw(0,0,PaintForm.ImageBack.Picture.Graphic);
    //PaintForm.PaintBox.Canvas.StretchDraw(fRct,PaintForm.PaintBox.Canvas);

  end;

end;


{***********************************************************}

procedure TSelectionRectangle.Reset;
begin
  if fLastActionType = atLine then ResetRotate else
  begin
    fSelecting := false;
    DisableTimer;
    initSelect;
    fcanvas.rectangle(fRct.Left, fRct.Top, fRct.Right, fRct.Bottom);
    fRct := Rect(0, 0, 0, 0);
    fSavedRct := fRct;
  end;
end;

procedure TSelectionRectangle.ResetRotate;
begin

  fSelecting := false;
  //DisableTimer;
  initLine;
  fcanvas.MoveTo(fRct.Left, fRct.Top);
  fcanvas.Lineto(fRct.Right, fRct.Bottom);
  fRct := Rect(0, 0, 0, 0);
  fSavedRct := fRct;

end;

///New Image

procedure TSelectionRectangle.Hide;
begin

  //DisableTimer;
  fcanvas.rectangle(fRct.Left, fRct.Top, fRct.Right, fRct.Bottom);
  //PaintForm.StatusBar.Panels[1].Text :='';
end;

procedure TSelectionRectangle.Show;
begin
  fCanvas.pen.style := psDot;
  fCanvas.pen.mode := pmNotXor;
  fCanvas.pen.width := 1;
  fCanvas.Brush.Style:=bsClear; //<---Add this line
  fCanvas.rectangle(fRct.Left, fRct.Top, fRct.Right, fRct.Bottom);
  //EnableTimer;
end;

///NewImage;

{***********************************************************}


procedure TSelectionRectangle.TimerProc;
begin
  fcanvas.Start;
  fcanvas.pen.Style := psSolid;
  fcanvas.rectangle(fRct.Left, fRct.Top, fRct.Right, fRct.Bottom);
  fTimerToggle := not fTimerToggle;
  fcanvas.Stop;
end;


procedure TSelectionRectangle.TimerEvent(Sender: TObject);
begin
  TimerProc;
end;

procedure TSelectionRectangle.DisableTimer;
begin

  fTim.Enabled := false;
  if fTimerToggle then
  begin
    TimerProc;
  end;
  initselect;
  fTimerToggle := false;

end;


procedure TSelectionRectangle.EnableTimer;
begin
  fTim.enabled := true;
end;

{***********************************************************}

function TSelectionRectangle.AdjustRect(theRect: TRect): TRect;
begin
  Result := theRect;
  if theRect.Left > theRect.Right then
  begin
    Result.Left := theRect.Right;
    Result.Right := theRect.Left;
  end;

  if theRect.Top > theRect.Bottom then
  begin
    Result.Top := theRect.Bottom;
    Result.Bottom := theRect.Top;
  end;

end;

function TSelectionRectangle.InsideRect(X, Y: integer; theRect: TRect; Tolerance: integer): boolean;
begin
  theRect := AdjustRect(theRect);
  if ((x > theRect.Left - Tolerance) and (x < theRect.Right + Tolerance) and
    (y > theRect.Top - Tolerance) and (y < theRect.Bottom + Tolerance)) then
    Result := true else
    Result := false;
end;


function TSelectionRectangle.GetResizeStyle(X, Y, Tolerance: integer): TResizeStyle;
begin

  Result := rssNone;

  if InsideRect(X, Y, fRct, 0) then Result := rssMove;

  if InsideRect(X, Y, Rect(fRct.Right - Tolerance, fRct.Top,
    fRct.Right + Tolerance, fRct.Bottom), 0) then Result := rssRight;

  if InsideRect(X, Y, Rect(fRct.Left - Tolerance, fRct.Top,
    fRct.Left + Tolerance, fRct.Bottom), 0) then Result := rssLeft;

  if InsideRect(X, Y, Rect(fRct.Left, fRct.Top - Tolerance,
    fRct.Right, fRct.Top + Tolerance), 0) then Result := rssTop;

  if InsideRect(X, Y, Rect(fRct.Left, fRct.Bottom - Tolerance,
    fRct.Right, fRct.Bottom + Tolerance), 0) then Result := rssBottom;

  if InsideRect(X, Y, Rect(fRct.Right - Tolerance, fRct.Bottom - Tolerance,
    fRct.Right + Tolerance, fRct.Bottom + Tolerance), 0) then Result := rssBottomRight;

  if InsideRect(X, Y, Rect(fRct.Left - Tolerance, fRct.Bottom - Tolerance,
    fRct.Left + Tolerance, fRct.Bottom + Tolerance), 0) then Result := rssBottomLeft;

  if InsideRect(X, Y, Rect(fRct.Right - Tolerance, fRct.Top - Tolerance,
    fRct.Right + Tolerance, fRct.Top + Tolerance), 0) then Result := rssTopRight;

  if InsideRect(X, Y, Rect(fRct.Left - Tolerance, fRct.Top - Tolerance,
    fRct.Left + Tolerance, fRct.Top + Tolerance), 0) then Result := rssTopLeft;
end;

function TSelectionRectangle.GetResizeCursor(X, Y, Tolerance: integer): TCursor;
var st: TResizeStyle;
begin
  result := crDefault;
  st := GetResizeStyle(X, Y, Tolerance);
  {if st = rssnone then Result := crCross2
  else }
  if st = rssmove then Result := crSizeAll
  else if (st = rssleft) or (st = rssright) then Result := crSizeWE
  else if (st = rssbottom) or (st = rsstop) then Result := crSizeNS
  else if (st = rssbottomLeft) or (st = rsstopRight) then Result := crSizeNESW
  else if (st = rssbottomRight) or (st = rsstopLeft) then Result := crSizeNWSE
  else result := crDefault;
end;

{***********************************************************}

procedure TSelectionRectangle.Draw(NewImage: TBitmap; w, h: integer; DoReset: Boolean);
begin
  if (not DoReset) then Hide else Reset;
  fCanvas.Draw(0, 0, NewImage);
  if (not DoReset) then Show;
end;

end.
